%--------------------------------------------------------+
% 3-d diffusion of one component in a sphere using       |
% the crank-nicolson-scheme + dirichlet boundary cond.   |
% The spatial grid is variable.                          |
% at i=0: Cim1=Cip1 and at i=nrhr: Cip1=Cim1             |
% Fred Gaidies, Sept 2021                                |
%--------------------------------------------------------+

%F.G., Sept 2021: No need to edit this script; ALL the required input needs
%to be entered in the Command Window. Before you run the script, the
%initial conditition (i.e., compositional core-to-rim profile) needs to be
%imported. An example file (Ydiff) is included - you would need to import
%the content as column vectors. It may be easiest to adjust this file so
%that the zoning fits your needs. The default setting uses kinetic data
%suggested for diffusion of Y + REE in garnet.

%This script creates the file profiles.mat which can be edited with text
%editors and spreadsheet software. The file contains the compositional
%profiles from core to rim, with the first column containing the duration
%of diffusion (in years) and the first row containing the radius (in cm).
%The file will be replaced during each run.

home

disp('Required input: Xnodecm and Xy. Note that the shell thickness is assumed to be constant. ')
TC       = input('Please enter the temperature [degree C]. ');
CFO2     = input('Please enter f(O2). The default is 1e-25, i.e., equilibration with graphite. To accept the default, please press the "return" key. ');
if isempty(CFO2)
        CFO2  =   1e-25;
end
disp(CFO2);
TT       = input('Please enter the total time of diffusion [My]. ');
IC       = input('Please enter the spacing of isochrones (coloured lines in plot) [years]. ');

%--initial condition-- imported
ST       = Xnodecm(3)-Xnodecm(2);          %this assumes that every shell is as thick as the third shell..
drV      = ones(1,max(size(Xnodecm)))*ST;  %distance vector, with a fixed resolution
% %---
nr       = max(size(drV));
Y        = Xy*100;
%----------------------------------------
length   = 0.5*drV(1)+sum(drV(2:end-1))+drV(end)/2;%length of model (cm), i.e. does not equal the radius
for i=1:nr-1
    X1(i)= drV(1)/2+sum(drV(2:i))+drV(i+1)/2;
end
X        = [0,X1];
X        = X+drV(1)/2;
CV       = Y;
cv       = CV(1:nr);
CVinit   = CV;
%--------------------------------------------------------------------------
year     = 365.25*24*60*60;
Dzero    = input('Please enter the pre-exponential constant D_0 [cm^2/s]. The default is 10^3 cm^2/year. ');
if isempty(Dzero)
        Dzero  =   1*10^3/year;
end
disp(Dzero);
%==========================================================================
TK       = TC+273.15;
RJ       = 8.31451070;    %universal gas constant in J/(mol*K)
RCal     = RJ/4.1868;     %universal gas constant in cal/(mol*K)
E_A      = input('Please enter the activation energy E_A [cal/mol]. The default is 300 kJ/mol. ');
if isempty(E_A)
        E_A  =   300*0.239006*1000;
end
disp(E_A);
Dtrace   = Dzero*exp((-(E_A))/(RCal*TK));
%--------------------------------------------------------------------------
%use this if fO2 not buffered by equilibration with graphite:
FugacityRatio = CFO2/1e-25;   %current fO2 over fO2 buffered with graphite
Dtrace = exp(log(Dtrace)+1/6*log(FugacityRatio)); %after Chakra+Gangu (1990, p. 142, equ.2)
%--------------------------------------------------------------------------
K    = 0.1*(min(drV))^2/max(Dtrace);%dt is 5-times smaller than the stability criterion for explecit schemes
'For appropriate accuracy use a smaller value than the following maximum time step size (in years):',num2str(K/year),
TSS  = input('Please enter the time step size [years]. For quick tests, enter a larger value here, and decrease it if the modelling turns out to be useful. ');
dt   = year*TSS;                             %size of timestep in seconds
nt   = year*1000000*TT;                      %timespan in seconds
n    = dt/drV(1)^2;
p    = dt/drV(end)^2;
%--------------------------------------------------------------------------
tic
%The equation system: A*cnew=M*cold

YA = zeros(nr,nr);  YM = zeros(nr,nr);

for i = 1:nr-2
    
    dr1 = drV(i)/2+drV(i+1)/2;
    dr2 = drV(i+1)/2+drV(i+2)/2;
    r   = sum(drV(1:i+1));
    l   = dt/(r*dr2*dr1*(dr2+dr1));
    
    YA(i+1,i)   =  l*Dtrace*(dr2^2-r*dr2);
    YA(i+1,i+1) =  1+l*Dtrace*(r*dr2+r*dr1-dr2^2+dr1^2);
    YA(i+1,i+2) =  l*Dtrace*(-dr1^2-r*dr1);

    YM(i+1,i)   =  -l*Dtrace*(dr2^2-r*dr2);
    YM(i+1,i+1) =  1-l*Dtrace*(r*dr2+r*dr1-dr2^2+dr1^2);
    YM(i+1,i+2) =  -l*Dtrace*(-dr1^2-r*dr1);
    
end
%boundary conditions-------------------------------------------------------
YA(1,1:2)       =  [1+3*Dtrace*n -3*Dtrace*n];
YA(nr,nr-1:nr)  =  [0 1];
YM(1,1:2)       =  [1-3*Dtrace*n 3*Dtrace*n];
YM(nr,nr-1:nr)  =  [0 1];
%--------------------------------------------------------------------------
A = sparse(YA);
M = sparse(YM);
%==========================================================================
%time loop
figure;clf;hold on;grid on;
plot(X,CVinit(1:max(size(Y))),'-g')
plot_tstep  =   [];
Y_all       =   [];
for tstep = 0:dt:nt
    Rhs         =   M*cv;
    cv          =   A\Rhs;
    cY          =   cv(1:nr);
    CV  = cv;
    if mod(tstep,year*IC)==0
        plot_tstep  =   [plot_tstep;tstep];
        Y_all       =   [Y_all;cY];
        CV          =   cv;
        plot(X,cv(1:nr));axis([0 sum(drV) min(Y) max(Y)]);
        drawnow;
    end
    xlabel('Radius [cm]');
    ylabel('Concentration of yttrium [ppm]');
    plot(X,CVinit(1:max(size(Y))),'-g')
end

Y_all_plot          =   zeros(max(size(plot_tstep)),max(size(drV)));

figure
hold on
xlabel('Radius [cm]');
ylabel('Concentration of yttrium [ppm]');
for i=1:max(size(plot_tstep))
plot(X,Y_all((i-1)*nr+1:i*nr))
hold on
Y_all_plot(i,:)    =   Y_all((i-1)*nr+1:i*nr);
end
plot(X,CVinit(1:nr),'-g')

profiles           =   [nan,X;plot_tstep/year,Y_all_plot];

save('profiles.mat','profiles','-ASCII');

toc