%--------------------------------------------------------+
% 3-d diffusion of four components in a sphere using     |
% the crank-nicolson-scheme, dirichlet boundary cond.,   |
% and a concentration-dependent diffusion-matrix         |
% (and a ). The spatial grid is variable.               |
% at i=0: Cim1=Cip1 and at i=nrhr: Cip1=Cim1             |
% Fred Gaidies, University of Basel, 10/06               |
%--------------------------------------------------------+

%F.G., July 2020: No need to edit this script; ALL the required input needs
%to be entered in the Command Window. Before you run the script, the
%initial conditition (i.e., compositional core-to-rim profile) needs to be
%imported. By default, a Theria_G output file is used. An example file
%(garnet_gen001.txt) is included - you would need to import the last 173
%lines as column vectors. It may be easiest to adjust this file so that the
%zoning fits your needs.

%This script creates the files spss_profiles.mat,
%alm_profiles.mat, grs_profiles.mat, and py_profiles.mat which can be
%edited with text editors and spreadsheet software. These files contain the
%compositional profiles from core to rim, with the first column containing
%the duration of diffusion (in years) and the first row containing the
%radius (in cm). The files will be replaced during each run.

%This model is part of Theria_G (Gaidies et al., 2008), so please refer to
%it if this script turns out to be useful for you. Note that this version
%correctly calculates the pressure-dependence of the energy barrier (as
%pointed out by Hugo Dominguez, Bern). Happy modelling!

home

disp('Required input (may be imported from Theria_G output): Xnodecm, Xmn, Xfe, Xmg. Note that the shell thickness is assumed to be constant. ')
TC       = input('Please enter the temperature [degree C]. ');
P        = input('Please enter the pressure [kbar]. ');
CFO2     = input('Please enter f(O2). The default is 1e-25, i.e., equilibration with graphite. To accept the default, please press the "return" key. ');
if isempty(CFO2)
        CFO2  =   1e-25;disp(CFO2);
end
TT       = input('Please enter the total time of diffusion [My]. ');
IC       = input('Please enter the spacing of isochrones (coloured lines in plot) [years]. ');

%--initial condition--Xnodecm, XMn, etc. imported from theriag output or similar--
ST       = Xnodecm(3)-Xnodecm(2);          %this assumes that every shell is as thick as the third shell..
drV      = ones(1,max(size(Xnodecm)))*ST;  %distance vector, with a fixed resolution
nr = max(size(drV));
Mn = Xmn*100;
Fe = Xfe*100;
Mg = Xmg*100;
Ca = 100-(Mn+Fe+Mg);
%----------------------------------------
length   = 0.5*drV(1)+sum(drV(2:end-1))+drV(end)/2;%length of model (cm), i.e. does not equal the radius
for i=1:nr-1
    X1(i)= drV(1)/2+sum(drV(2:i))+drV(i+1)/2;
end
X        = [0,X1];
X        = X+drV(1)/2;
CV       = [Mn;Fe;Mg;Ca];
cv       = CV(1:3*nr);
CVinit   = CV;
%--------------------------------------------------------------------------
%kinetic parameters from Chakra + Gangu (1992) at fO2 defined by graphite buffer
DMnzero  = 5.1*10^-4;       %pre-exponential constant in cm^2/s
AEMnCal  = 60569;           %activation energy for diffusion at 1 bar in cal/mol
AEMn     = AEMnCal*4.1868;  %activation energy for diffusion at 1 bar in J/mol
AVMn     = 6.0;             %activation volume for diffusion in cm^3/mol
DFezero  = 6.4*10^-4;
AEFeCal  = 65824;
AEFe     = AEFeCal*4.1868; 
AVFe     = 5.6;
DMgzero  = 1.1*10^-3;
AEMgCal  = 67997;
AEMg     = AEMgCal*4.1868;
AVMg     = 5.3;
%==========================================================================
TK       = TC+273.15;
RJ       = 8.31451070;    %universal gas constant in J/(mol*K)
%RCal     = RJ/4.1868;     %universal gas constant in cal/(mol*K)
DMntrace = DMnzero*exp((-AEMn-(100*(P-0.001))*AVMn)/(RJ*TK));
DFetrace = DFezero*exp((-AEFe-(100*(P-0.001))*AVFe)/(RJ*TK));
DMgtrace = DMgzero*exp((-AEMg-(100*(P-0.001))*AVMg)/(RJ*TK));
DCatrace = 0.5*DFetrace;  %after Loomis (?) and Florence & Spear (1991)
%--------------------------------------------------------------------------
%use this if fO2 not buffered by equilibration with graphite:
FugacityRatio = CFO2/1e-25;   %current fO2 over fO2 buffered with graphite
DMntrace = exp(log(DMntrace)+1/6*log(FugacityRatio)); %after Chakra+Gangu (1990, p. 142, equ.2)
DFetrace = exp(log(DFetrace)+1/6*log(FugacityRatio));
DMgtrace = exp(log(DMgtrace)+1/6*log(FugacityRatio));
DCatrace = exp(log(DCatrace)+1/6*log(FugacityRatio));
%calculation of the D-matrix after Lasaga (1979)...
%assuming negligible activity coefficient gradients;
DMnMn = ones(nr,1);DMnFe = ones(nr,1);DMnMg = ones(nr,1);
DFeMn = ones(nr,1);DFeFe = ones(nr,1);DFeMg = ones(nr,1);
DMgMn = ones(nr,1);DMgFe = ones(nr,1);DMgMg = ones(nr,1);

for    i = 1:nr
    denom      = CV(i)/100*DMntrace+CV(nr+i)/100 ...
        *DFetrace+CV(2*nr+i)/100*DMgtrace+CV(3*nr+i)/100*DCatrace;
    DMnMn(i,1) = DMntrace*1-((DMntrace*CV(0*nr+i)/100)/denom)*(DMntrace-DCatrace);
    DMnFe(i,1) = DMntrace*0-((DMntrace*CV(0*nr+i)/100)/denom)*(DFetrace-DCatrace);
    DMnMg(i,1) = DMntrace*0-((DMntrace*CV(0*nr+i)/100)/denom)*(DMgtrace-DCatrace);
    DFeMn(i,1) = DFetrace*0-((DFetrace*CV(1*nr+i)/100)/denom)*(DMntrace-DCatrace);
    DFeFe(i,1) = DFetrace*1-((DFetrace*CV(1*nr+i)/100)/denom)*(DFetrace-DCatrace);
    DFeMg(i,1) = DFetrace*0-((DFetrace*CV(1*nr+i)/100)/denom)*(DMgtrace-DCatrace);
    DMgMn(i,1) = DMgtrace*0-((DMgtrace*CV(2*nr+i)/100)/denom)*(DMntrace-DCatrace);
    DMgFe(i,1) = DMgtrace*0-((DMgtrace*CV(2*nr+i)/100)/denom)*(DFetrace-DCatrace);
    DMgMg(i,1) = DMgtrace*1-((DMgtrace*CV(2*nr+i)/100)/denom)*(DMgtrace-DCatrace);
end
%--------------------------------------------------------------------------
year = 365.25*24*60*60;
K    = 0.1*(min(drV))^2/max(DMnMn);%dt is 5-times smaller than the stability criterion for explecit schemes
'For appropriate accuracy, use a smaller value than the following maximum time step size [years]:',num2str(K/year),
TSS  = input('Please enter the time step size [years]. For quick tests, enter a larger value here, and decrease it if the modelling turns out to be useful. ');
dt   = year*TSS;                             %size of timestep in seconds
nt   = year*1000000*TT;                      %timespan in seconds
n    = dt/drV(1)^2;
p    = dt/drV(end)^2;
%--------------------------------------------------------------------------
tic
%The equation system: A*cnew=M*cold

MnMnA = zeros(nr,nr);  MnMnM = zeros(nr,nr);
MnFeA = zeros(nr,nr);  MnFeM = zeros(nr,nr);
MnMgA = zeros(nr,nr);  MnMgM = zeros(nr,nr);
FeMnA = zeros(nr,nr);  FeMnM = zeros(nr,nr);
FeFeA = zeros(nr,nr);  FeFeM = zeros(nr,nr);
FeMgA = zeros(nr,nr);  FeMgM = zeros(nr,nr);
MgMnA = zeros(nr,nr);  MgMnM = zeros(nr,nr);
MgFeA = zeros(nr,nr);  MgFeM = zeros(nr,nr);
MgMgA = zeros(nr,nr);  MgMgM = zeros(nr,nr);

for i = 1:nr-2
    
    dr1 = drV(i)/2+drV(i+1)/2;
    dr2 = drV(i+1)/2+drV(i+2)/2;
    r   = sum(drV(1:i+1));
    l   = dt/(r*dr2*dr1*(dr2+dr1));
    
    D11 = DMnMn(i+1); D12 = DMnFe(i+1); D13 = DMnMg(i+1);
    D21 = DFeMn(i+1); D22 = DFeFe(i+1); D23 = DFeMg(i+1);
    D31 = DMgMn(i+1); D32 = DMgFe(i+1); D33 = DMgMg(i+1);
    
    MnMnA(i+1,i)   =  l*D11*(dr2^2-r*dr2);
    MnMnA(i+1,i+1) =  1+l*D11*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnMnA(i+1,i+2) =  l*D11*(-dr1^2-r*dr1);

    MnMnM(i+1,i)   =  -l*D11*(dr2^2-r*dr2);
    MnMnM(i+1,i+1) =  1-l*D11*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnMnM(i+1,i+2) =  -l*D11*(-dr1^2-r*dr1);

    MnFeA(i+1,i)   =  l*D12*(dr2^2-r*dr2);
    MnFeA(i+1,i+1) =  l*D12*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnFeA(i+1,i+2) =  l*D12*(-dr1^2-r*dr1);

    MnFeM(i+1,i)   =  -l*D12*(dr2^2-r*dr2);
    MnFeM(i+1,i+1) =  -l*D12*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnFeM(i+1,i+2) =  -l*D12*(-dr1^2-r*dr1);

    MnMgA(i+1,i)   =  l*D13*(dr2^2-r*dr2);
    MnMgA(i+1,i+1) =  l*D13*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnMgA(i+1,i+2) =  l*D13*(-dr1^2-r*dr1);

    MnMgM(i+1,i)   =  -l*D13*(dr2^2-r*dr2);
    MnMgM(i+1,i+1) =  -l*D13*(r*dr2+r*dr1-dr2^2+dr1^2);
    MnMgM(i+1,i+2) =  -l*D13*(-dr1^2-r*dr1);

    FeMnA(i+1,i)   =  l*D21*(dr2^2-r*dr2);
    FeMnA(i+1,i+1) =  l*D21*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeMnA(i+1,i+2) =  l*D21*(-dr1^2-r*dr1);

    FeMnM(i+1,i)   =  -l*D21*(dr2^2-r*dr2);
    FeMnM(i+1,i+1) =  -l*D21*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeMnM(i+1,i+2) =  -l*D21*(-dr1^2-r*dr1);

    FeFeA(i+1,i)   =  l*D22*(dr2^2-r*dr2);
    FeFeA(i+1,i+1) =  1+l*D22*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeFeA(i+1,i+2) =  l*D22*(-dr1^2-r*dr1);

    FeFeM(i+1,i)   =  -l*D22*(dr2^2-r*dr2);
    FeFeM(i+1,i+1) =  1-l*D22*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeFeM(i+1,i+2) =  -l*D22*(-dr1^2-r*dr1);

    FeMgA(i+1,i)   =  l*D23*(dr2^2-r*dr2);
    FeMgA(i+1,i+1) =  l*D23*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeMgA(i+1,i+2) =  l*D23*(-dr1^2-r*dr1);

    FeMgM(i+1,i)   =  -l*D23*(dr2^2-r*dr2);
    FeMgM(i+1,i+1) =  -l*D23*(r*dr2+r*dr1-dr2^2+dr1^2);
    FeMgM(i+1,i+2) =  -l*D23*(-dr1^2-r*dr1);

    MgMnA(i+1,i)   =  l*D31*(dr2^2-r*dr2);
    MgMnA(i+1,i+1) =  l*D31*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgMnA(i+1,i+2) =  l*D31*(-dr1^2-r*dr1);

    MgMnM(i+1,i)   =  -l*D31*(dr2^2-r*dr2);
    MgMnM(i+1,i+1) =  -l*D31*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgMnM(i+1,i+2) =  -l*D31*(-dr1^2-r*dr1);

    MgFeA(i+1,i)   =  l*D32*(dr2^2-r*dr2);
    MgFeA(i+1,i+1) =  l*D32*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgFeA(i+1,i+2) =  l*D32*(-dr1^2-r*dr1);

    MgFeM(i+1,i)   =  -l*D32*(dr2^2-r*dr2);
    MgFeM(i+1,i+1) =  -l*D32*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgFeM(i+1,i+2) =  -l*D32*(-dr1^2-r*dr1);

    MgMgA(i+1,i)   =  l*D33*(dr2^2-r*dr2);
    MgMgA(i+1,i+1) =  1+l*D33*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgMgA(i+1,i+2) =  l*D33*(-dr1^2-r*dr1);

    MgMgM(i+1,i)   =  -l*D33*(dr2^2-r*dr2);
    MgMgM(i+1,i+1) =  1-l*D33*(r*dr2+r*dr1-dr2^2+dr1^2);
    MgMgM(i+1,i+2) =  -l*D33*(-dr1^2-r*dr1);
    
end
%boundary conditions-------------------------------------------------------
MnMnA(1,1:2)       =  [1+3*DMnMn(1)*n -3*DMnMn(1)*n];
MnMnA(nr,nr-1:nr)  =  [0 1];
MnMnM(1,1:2)       =  [1-3*DMnMn(1)*n 3*DMnMn(1)*n];
MnMnM(nr,nr-1:nr)  =  [0 1];

MnFeA(1,1:2)       =  [3*DMnFe(1)*n -3*DMnFe(1)*n];
MnFeA(nr,nr-1:nr)  =  [0 0];
MnFeM(1,1:2)       =  [-3*DMnFe(1)*n 3*DMnFe(1)*n];
MnFeM(nr,nr-1:nr)  =  [0 0];

MnMgA(1,1:2)       =  [3*DMnMg(1)*n -3*DMnMg(1)*n];
MnMgA(nr,nr-1:nr)  =  [0 0];
MnMgM(1,1:2)       =  [-3*DMnMg(1)*n 3*DMnMg(1)*n];
MnMgM(nr,nr-1:nr)  =  [0 0];

FeMnA(1,1:2)       =  [3*DFeMn(1)*n -3*DFeMn(1)*n];
FeMnA(nr,nr-1:nr)  =  [0 0];
FeMnM(1,1:2)       =  [-3*DFeMn(1)*n 3*DFeMn(1)*n];
FeMnM(nr,nr-1:nr)  =  [0 0];

FeFeA(1,1:2)       =  [1+3*DFeFe(1)*n -3*DFeFe(1)*n];
FeFeA(nr,nr-1:nr)  =  [0 1];
FeFeM(1,1:2)       =  [1-3*DFeFe(1)*n 3*DFeFe(1)*n];
FeFeM(nr,nr-1:nr)  =  [0 1];

FeMgA(1,1:2)       =  [3*DFeMg(1)*n -3*DFeMg(1)*n];
FeMgA(nr,nr-1:nr)  =  [0 0];
FeMgM(1,1:2)       =  [-3*DFeMg(1)*n 3*DFeMg(1)*n];
FeMgM(nr,nr-1:nr)  =  [0 0];

MgMnA(1,1:2)       =  [3*DMgMn(1)*n -3*DMgMn(1)*n];
MgMnA(nr,nr-1:nr)  =  [0 0];
MgMnM(1,1:2)       =  [-3*DMgMn(1)*n 3*DMgMn(1)*n];
MgMnM(nr,nr-1:nr)  =  [0 0];

MgFeA(1,1:2)       =  [3*DMgFe(1)*n -3*DMgFe(1)*n];
MgFeA(nr,nr-1:nr)  =  [0 0];
MgFeM(1,1:2)       =  [-3*DMgFe(1)*n 3*DMgFe(1)*n];
MgFeM(nr,nr-1:nr)  =  [0 0];

MgMgA(1,1:2)       =  [1+3*DMgMg(1)*n -3*DMgMg(1)*n];
MgMgA(nr,nr-1:nr)  =  [0 1];
MgMgM(1,1:2)       =  [1-3*DMgMg(1)*n 3*DMgMg(1)*n];
MgMgM(nr,nr-1:nr)  =  [0 1];
%--------------------------------------------------------------------------
A = sparse([MnMnA,MnFeA,MnMgA;FeMnA,FeFeA,FeMgA;MgMnA,MgFeA,MgMgA]);
M = sparse([MnMnM,MnFeM,MnMgM;FeMnM,FeFeM,FeMgM;MgMnM,MgFeM,MgMgM]);
%==========================================================================
%time loop
figure;clf;hold on;grid on;
plot(X,CVinit(1:max(size(Mn))),'-g')
plot_tstep  =   [];
Mn_all      =   [];
Fe_all      =   [];
Mg_all      =   [];
Ca_all      =   [];
for tstep = 0:dt:nt
    Rhs         =   M*cv;
    cv          =   A\Rhs;
    cMn         =   cv(1:nr);
    cFe         =   cv(nr+1:2*nr);
    cMg         =   cv(2*nr+1:3*nr);
    cCa         =   100-cMn-cFe-cMg;
    CV  = [cv;cCa];
    if mod(tstep,year*IC)==0
        plot_tstep  =   [plot_tstep;tstep];
        Mn_all      =   [Mn_all;cMn];
        Fe_all      =   [Fe_all;cFe];
        Mg_all      =   [Mg_all;cMg];
        Ca_all      =   [Ca_all;cCa];
        CV  = [cv;cCa];
        plot(X,cv(1:nr));axis([0 sum(drV) min(Mn) max(Mn)]);
        drawnow;
    end
    xlabel('Radius [cm]');
    ylabel('Concentration of spessartine  [mol-%]');
    plot(X,CVinit(1:max(size(Mn))),'-g')
    %====================================================================
    %UPDATE of A and M
    DMnMn = ones(nr,1);DMnFe = ones(nr,1);DMnMg = ones(nr,1);
    DFeMn = ones(nr,1);DFeFe = ones(nr,1);DFeMg = ones(nr,1);
    DMgMn = ones(nr,1);DMgFe = ones(nr,1);DMgMg = ones(nr,1);
    for    i = 1:nr
        denom      = CV(i)/100*DMntrace+CV(nr+i)/100 ...
            *DFetrace+CV(2*nr+i)/100*DMgtrace+CV(3*nr+i)/100*DCatrace;
        DMnMn(i,1) = DMntrace*1-((DMntrace*CV(0*nr+i)/100)/denom)*(DMntrace-DCatrace);
        DMnFe(i,1) = DMntrace*0-((DMntrace*CV(0*nr+i)/100)/denom)*(DFetrace-DCatrace);
        DMnMg(i,1) = DMntrace*0-((DMntrace*CV(0*nr+i)/100)/denom)*(DMgtrace-DCatrace);
        DFeMn(i,1) = DFetrace*0-((DFetrace*CV(1*nr+i)/100)/denom)*(DMntrace-DCatrace);
        DFeFe(i,1) = DFetrace*1-((DFetrace*CV(1*nr+i)/100)/denom)*(DFetrace-DCatrace);
        DFeMg(i,1) = DFetrace*0-((DFetrace*CV(1*nr+i)/100)/denom)*(DMgtrace-DCatrace);
        DMgMn(i,1) = DMgtrace*0-((DMgtrace*CV(2*nr+i)/100)/denom)*(DMntrace-DCatrace);
        DMgFe(i,1) = DMgtrace*0-((DMgtrace*CV(2*nr+i)/100)/denom)*(DFetrace-DCatrace);
        DMgMg(i,1) = DMgtrace*1-((DMgtrace*CV(2*nr+i)/100)/denom)*(DMgtrace-DCatrace);
    end
    %======================================================================
    MnMnA = zeros(nr,nr);  MnMnM = zeros(nr,nr);
    MnFeA = zeros(nr,nr);  MnFeM = zeros(nr,nr);
    MnMgA = zeros(nr,nr);  MnMgM = zeros(nr,nr);
    FeMnA = zeros(nr,nr);  FeMnM = zeros(nr,nr);
    FeFeA = zeros(nr,nr);  FeFeM = zeros(nr,nr);
    FeMgA = zeros(nr,nr);  FeMgM = zeros(nr,nr);
    MgMnA = zeros(nr,nr);  MgMnM = zeros(nr,nr);
    MgFeA = zeros(nr,nr);  MgFeM = zeros(nr,nr);
    MgMgA = zeros(nr,nr);  MgMgM = zeros(nr,nr);

    for i = 1:nr-2

        dr1 = drV(i)/2+drV(i+1)/2;
        dr2 = drV(i+1)/2+drV(i+2)/2;
        r   = sum(drV(1:i+1));
        l   = dt/(r*dr2*dr1*(dr2+dr1));

        D11 = DMnMn(i+1); D12 = DMnFe(i+1); D13 = DMnMg(i+1);
        D21 = DFeMn(i+1); D22 = DFeFe(i+1); D23 = DFeMg(i+1);
        D31 = DMgMn(i+1); D32 = DMgFe(i+1); D33 = DMgMg(i+1);

        MnMnA(i+1,i)   =  l*D11*(dr2^2-r*dr2);
        MnMnA(i+1,i+1) =  1+l*D11*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnMnA(i+1,i+2) =  l*D11*(-dr1^2-r*dr1);

        MnMnM(i+1,i)   =  -l*D11*(dr2^2-r*dr2);
        MnMnM(i+1,i+1) =  1-l*D11*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnMnM(i+1,i+2) =  -l*D11*(-dr1^2-r*dr1);

        MnFeA(i+1,i)   =  l*D12*(dr2^2-r*dr2);
        MnFeA(i+1,i+1) =  l*D12*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnFeA(i+1,i+2) =  l*D12*(-dr1^2-r*dr1);

        MnFeM(i+1,i)   =  -l*D12*(dr2^2-r*dr2);
        MnFeM(i+1,i+1) =  -l*D12*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnFeM(i+1,i+2) =  -l*D12*(-dr1^2-r*dr1);

        MnMgA(i+1,i)   =  l*D13*(dr2^2-r*dr2);
        MnMgA(i+1,i+1) =  l*D13*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnMgA(i+1,i+2) =  l*D13*(-dr1^2-r*dr1);

        MnMgM(i+1,i)   =  -l*D13*(dr2^2-r*dr2);
        MnMgM(i+1,i+1) =  -l*D13*(r*dr2+r*dr1-dr2^2+dr1^2);
        MnMgM(i+1,i+2) =  -l*D13*(-dr1^2-r*dr1);

        FeMnA(i+1,i)   =  l*D21*(dr2^2-r*dr2);
        FeMnA(i+1,i+1) =  l*D21*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeMnA(i+1,i+2) =  l*D21*(-dr1^2-r*dr1);

        FeMnM(i+1,i)   =  -l*D21*(dr2^2-r*dr2);
        FeMnM(i+1,i+1) =  -l*D21*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeMnM(i+1,i+2) =  -l*D21*(-dr1^2-r*dr1);

        FeFeA(i+1,i)   =  l*D22*(dr2^2-r*dr2);
        FeFeA(i+1,i+1) =  1+l*D22*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeFeA(i+1,i+2) =  l*D22*(-dr1^2-r*dr1);

        FeFeM(i+1,i)   =  -l*D22*(dr2^2-r*dr2);
        FeFeM(i+1,i+1) =  1-l*D22*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeFeM(i+1,i+2) =  -l*D22*(-dr1^2-r*dr1);

        FeMgA(i+1,i)   =  l*D23*(dr2^2-r*dr2);
        FeMgA(i+1,i+1) =  l*D23*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeMgA(i+1,i+2) =  l*D23*(-dr1^2-r*dr1);

        FeMgM(i+1,i)   =  -l*D23*(dr2^2-r*dr2);
        FeMgM(i+1,i+1) =  -l*D23*(r*dr2+r*dr1-dr2^2+dr1^2);
        FeMgM(i+1,i+2) =  -l*D23*(-dr1^2-r*dr1);

        MgMnA(i+1,i)   =  l*D31*(dr2^2-r*dr2);
        MgMnA(i+1,i+1) =  l*D31*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgMnA(i+1,i+2) =  l*D31*(-dr1^2-r*dr1);

        MgMnM(i+1,i)   =  -l*D31*(dr2^2-r*dr2);
        MgMnM(i+1,i+1) =  -l*D31*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgMnM(i+1,i+2) =  -l*D31*(-dr1^2-r*dr1);

        MgFeA(i+1,i)   =  l*D32*(dr2^2-r*dr2);
        MgFeA(i+1,i+1) =  l*D32*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgFeA(i+1,i+2) =  l*D32*(-dr1^2-r*dr1);

        MgFeM(i+1,i)   =  -l*D32*(dr2^2-r*dr2);
        MgFeM(i+1,i+1) =  -l*D32*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgFeM(i+1,i+2) =  -l*D32*(-dr1^2-r*dr1);

        MgMgA(i+1,i)   =  l*D33*(dr2^2-r*dr2);
        MgMgA(i+1,i+1) =  1+l*D33*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgMgA(i+1,i+2) =  l*D33*(-dr1^2-r*dr1);

        MgMgM(i+1,i)   =  -l*D33*(dr2^2-r*dr2);
        MgMgM(i+1,i+1) =  1-l*D33*(r*dr2+r*dr1-dr2^2+dr1^2);
        MgMgM(i+1,i+2) =  -l*D33*(-dr1^2-r*dr1);

    end
    %boundary conditions-------------------------------------------------------
    MnMnA(1,1:2)       =  [1+3*DMnMn(1)*n -3*DMnMn(1)*n];
    MnMnA(nr,nr-1:nr)  =  [0 1];
    MnMnM(1,1:2)       =  [1-3*DMnMn(1)*n 3*DMnMn(1)*n];
    MnMnM(nr,nr-1:nr)  =  [0 1];

    MnFeA(1,1:2)       =  [3*DMnFe(1)*n -3*DMnFe(1)*n];
    MnFeA(nr,nr-1:nr)  =  [0 0];
    MnFeM(1,1:2)       =  [-3*DMnFe(1)*n 3*DMnFe(1)*n];
    MnFeM(nr,nr-1:nr)  =  [0 0];

    MnMgA(1,1:2)       =  [3*DMnMg(1)*n -3*DMnMg(1)*n];
    MnMgA(nr,nr-1:nr)  =  [0 0];
    MnMgM(1,1:2)       =  [-3*DMnMg(1)*n 3*DMnMg(1)*n];
    MnMgM(nr,nr-1:nr)  =  [0 0];

    FeMnA(1,1:2)       =  [3*DFeMn(1)*n -3*DFeMn(1)*n];
    FeMnA(nr,nr-1:nr)  =  [0 0];
    FeMnM(1,1:2)       =  [-3*DFeMn(1)*n 3*DFeMn(1)*n];
    FeMnM(nr,nr-1:nr)  =  [0 0];

    FeFeA(1,1:2)       =  [1+3*DFeFe(1)*n -3*DFeFe(1)*n];
    FeFeA(nr,nr-1:nr)  =  [0 1];
    FeFeM(1,1:2)       =  [1-3*DFeFe(1)*n 3*DFeFe(1)*n];
    FeFeM(nr,nr-1:nr)  =  [0 1];

    FeMgA(1,1:2)       =  [3*DFeMg(1)*n -3*DFeMg(1)*n];
    FeMgA(nr,nr-1:nr)  =  [0 0];
    FeMgM(1,1:2)       =  [-3*DFeMg(1)*n 3*DFeMg(1)*n];
    FeMgM(nr,nr-1:nr)  =  [0 0];

    MgMnA(1,1:2)       =  [3*DMgMn(1)*n -3*DMgMn(1)*n];
    MgMnA(nr,nr-1:nr)  =  [0 0];
    MgMnM(1,1:2)       =  [-3*DMgMn(1)*n 3*DMgMn(1)*n];
    MgMnM(nr,nr-1:nr)  =  [0 0];

    MgFeA(1,1:2)       =  [3*DMgFe(1)*n -3*DMgFe(1)*n];
    MgFeA(nr,nr-1:nr)  =  [0 0];
    MgFeM(1,1:2)       =  [-3*DMgFe(1)*n 3*DMgFe(1)*n];
    MgFeM(nr,nr-1:nr)  =  [0 0];

    MgMgA(1,1:2)       =  [1+3*DMgMg(1)*n -3*DMgMg(1)*n];
    MgMgA(nr,nr-1:nr)  =  [0 1];
    MgMgM(1,1:2)       =  [1-3*DMgMg(1)*n 3*DMgMg(1)*n];
    MgMgM(nr,nr-1:nr)  =  [0 1];
    %--------------------------------------------------------------------------
    A = sparse([MnMnA,MnFeA,MnMgA;FeMnA,FeFeA,FeMgA;MgMnA,MgFeA,MgMgA]);
    M = sparse([MnMnM,MnFeM,MnMgM;FeMnM,FeFeM,FeMgM;MgMnM,MgFeM,MgMgM]);
    %--------------------------------------------------------------------------
end

Ca_all_plot     =   zeros(max(size(plot_tstep)),max(size(drV)));
Fe_all_plot     =   Ca_all_plot;
Mn_all_plot     =   Ca_all_plot;
Mg_all_plot     =   Ca_all_plot;

figure
hold on
xlabel('Radius [cm]');
ylabel('Concentration of almandine [mol-%]');
for i=1:max(size(plot_tstep))
plot(X,Fe_all((i-1)*nr+1:i*nr))
hold on
Fe_all_plot(i,:)    =   Fe_all((i-1)*nr+1:i*nr);
end
plot(X,CVinit(nr+1:2*nr),'-g')

figure
hold on
xlabel('Radius [cm]');
ylabel('Concentration of pyrope [mol-%]');
for i=1:max(size(plot_tstep))
plot(X,Mg_all((i-1)*nr+1:i*nr))
hold on
Mg_all_plot(i,:)    =   Mg_all((i-1)*nr+1:i*nr);
end
plot(X,CVinit(2*nr+1:3*nr),'-g')

figure
hold on
xlabel('Radius [cm]');
ylabel('Concentration of grossular [mol-%]');
for i=1:max(size(plot_tstep))
plot(X,Ca_all((i-1)*nr+1:i*nr))
hold on
Ca_all_plot(i,:)    =   Ca_all((i-1)*nr+1:i*nr);
end
plot(X,CVinit(3*nr+1:4*nr),'-g')

figure
hold on
xlabel('Radius [cm]');
ylabel('Concentration of spessartine [mol%]');
for i=1:max(size(plot_tstep))
plot(X,Mn_all((i-1)*nr+1:i*nr))
hold on
Mn_all_plot(i,:)    =   Mn_all((i-1)*nr+1:i*nr);
end
plot(X,CVinit(1:nr),'-g')

alm_profiles    =   [nan,X;plot_tstep/year,Fe_all_plot];
py_profiles     =   [nan,X;plot_tstep/year,Mg_all_plot];
grs_profiles    =   [nan,X;plot_tstep/year,Ca_all_plot];
spss_profiles   =   [nan,X;plot_tstep/year,Mn_all_plot];

save('alm_profiles.mat','alm_profiles','-ASCII','-tabs');
save('py_profiles.mat','py_profiles','-ASCII');
save('grs_profiles.mat','grs_profiles','-ASCII');
save('spss_profiles.mat','spss_profiles','-ASCII');

toc